@@ -11,6 +11,7 @@
 #include "log.h"
 #include "sway/config.h"
 #include "sway/ipc-json.h"
+#include "sway/server.h"
 #include "sway/tree/container.h"
 #include "sway/tree/view.h"
 #include "sway/tree/workspace.h"
@@ -154,7 +155,7 @@ static json_object *ipc_json_output_mode_description(
 	return mode_object;
 }
 
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 static const char *ipc_json_xwindow_type_description(struct sway_view *view) {
 	struct wlr_xwayland_surface *surface = view->wlr_xwayland_surface;
 	struct sway_xwayland *xwayland = &server.xwayland;
@@ -288,6 +289,8 @@ static json_object *ipc_json_create_node(int id, const char* type, char *name,
 	json_object_object_add(object, "focus", focus);
 	json_object_object_add(object, "fullscreen_mode", json_object_new_int(0));
 	json_object_object_add(object, "sticky", json_object_new_boolean(false));
+	json_object_object_add(object, "floating", NULL);
+	json_object_object_add(object, "scratchpad_state", NULL);
 
 	return object;
 }
@@ -396,6 +399,8 @@ static void ipc_json_describe_enabled_output(struct sway_output *output,
 	}
 
 	json_object_object_add(object, "max_render_time", json_object_new_int(output->max_render_time));
+
+	json_object_object_add(object, "allow_tearing", json_object_new_boolean(output->allow_tearing));
 }
 
 json_object *ipc_json_describe_disabled_output(struct sway_output *output) {
@@ -575,9 +580,10 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 	bool visible = view_is_visible(c->view);
 	json_object_object_add(object, "visible", json_object_new_boolean(visible));
 
+	bool has_titlebar = c->title_bar.tree->node.enabled;
 	struct wlr_box window_box = {
 		c->pending.content_x - c->pending.x,
-		(c->current.border == B_PIXEL) ? c->pending.content_y - c->pending.y : 0,
+		has_titlebar ? 0 : c->pending.content_y - c->pending.y,
 		c->pending.content_width,
 		c->pending.content_height
 	};
@@ -589,6 +595,8 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 
 	json_object_object_add(object, "max_render_time", json_object_new_int(c->view->max_render_time));
 
+	json_object_object_add(object, "allow_tearing", json_object_new_boolean(view_can_tear(c->view)));
+
 	json_object_object_add(object, "shell", json_object_new_string(view_get_shell(c->view)));
 
 	json_object_object_add(object, "inhibit_idle",
@@ -631,7 +639,7 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 			json_object_new_string(ipc_json_content_type_description(content_type)));
 	}
 
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 	if (c->view->type == SWAY_VIEW_XWAYLAND) {
 		json_object_object_add(object, "window",
 				json_object_new_int(view_get_x11_window_id(c->view)));
@@ -675,7 +683,8 @@ static void ipc_json_describe_view(struct sway_container *c, json_object *object
 static void ipc_json_describe_container(struct sway_container *c, json_object *object) {
 	json_object_object_add(object, "name",
 			c->title ? json_object_new_string(c->title) : NULL);
-	if (container_is_floating(c)) {
+	bool floating = container_is_floating(c);
+	if (floating) {
 		json_object_object_add(object, "type",
 				json_object_new_string("floating_con"));
 	}
@@ -693,9 +702,17 @@ static void ipc_json_describe_container(struct sway_container *c, json_object *o
 	json_object_object_add(object, "urgent", json_object_new_boolean(urgent));
 	json_object_object_add(object, "sticky", json_object_new_boolean(c->is_sticky));
 
+	// sway doesn't track the floating reason, so we can't use "auto_on" or "user_off"
+	json_object_object_add(object, "floating",
+			json_object_new_string(floating ? "user_on" : "auto_off"));
+
 	json_object_object_add(object, "fullscreen_mode",
 			json_object_new_int(c->pending.fullscreen_mode));
 
+	// sway doesn't track if window was resized in scratchpad, so we can't use "changed"
+	json_object_object_add(object, "scratchpad_state",
+			json_object_new_string(!c->scratchpad ? "none" : "fresh"));
+
 	struct sway_node *parent = node_get_parent(&c->node);
 	struct wlr_box parent_box = {0, 0, 0, 0};
 
@@ -979,6 +996,18 @@ static json_object *describe_libinput_device(struct libinput_device *device) {
 		}
 		json_object_object_add(object, "click_method",
 				json_object_new_string(click_method));
+
+		const char *button_map = "unknown";
+		switch (libinput_device_config_click_get_clickfinger_button_map(device)) {
+		case LIBINPUT_CONFIG_CLICKFINGER_MAP_LRM:
+			button_map = "lrm";
+			break;
+		case LIBINPUT_CONFIG_CLICKFINGER_MAP_LMR:
+			button_map = "lmr";
+			break;
+		}
+		json_object_object_add(object, "clickfinger_button_map",
+				json_object_new_string(button_map));
 	}
 
 	if (libinput_device_config_middle_emulation_is_available(device)) {
@@ -1086,10 +1115,6 @@ json_object *ipc_json_describe_input(struct sway_input_device *device) {
 		json_object_new_string(device->identifier));
 	json_object_object_add(object, "name",
 		json_object_new_string(device->wlr_device->name));
-	json_object_object_add(object, "vendor",
-		json_object_new_int(device->wlr_device->vendor));
-	json_object_object_add(object, "product",
-		json_object_new_int(device->wlr_device->product));
 	json_object_object_add(object, "type",
 		json_object_new_string(
 			input_device_get_type(device)));
@@ -1100,15 +1125,17 @@ json_object *ipc_json_describe_input(struct sway_input_device *device) {
 		struct xkb_keymap *keymap = keyboard->keymap;
 		struct xkb_state *state = keyboard->xkb_state;
 
-		json_object_object_add(object, "repeat_delay", 
+		json_object_object_add(object, "repeat_delay",
 			json_object_new_int(keyboard->repeat_info.delay));
-		json_object_object_add(object, "repeat_rate", 
+		json_object_object_add(object, "repeat_rate",
 			json_object_new_int(keyboard->repeat_info.rate));
 
 		json_object *layouts_arr = json_object_new_array();
 		json_object_object_add(object, "xkb_layout_names", layouts_arr);
 
-		xkb_layout_index_t num_layouts = xkb_keymap_num_layouts(keymap);
+		xkb_layout_index_t num_layouts =
+			keymap ? xkb_keymap_num_layouts(keymap) : 0;
+		// Virtual keyboards might have null keymap
 		xkb_layout_index_t layout_idx;
 		for (layout_idx = 0; layout_idx < num_layouts; layout_idx++) {
 			const char *layout = xkb_keymap_layout_get_name(keymap, layout_idx);
@@ -1143,6 +1170,10 @@ json_object *ipc_json_describe_input(struct sway_input_device *device) {
 		libinput_dev = wlr_libinput_get_device_handle(device->wlr_device);
 		json_object_object_add(object, "libinput",
 				describe_libinput_device(libinput_dev));
+		json_object_object_add(object, "vendor",
+			json_object_new_int(libinput_device_get_id_vendor(libinput_dev)));
+		json_object_object_add(object, "product",
+			json_object_new_int(libinput_device_get_id_product(libinput_dev)));
 	}
 #endif
 
